;==============================================================================
;
; MSX スターソルジャー
;
;==============================================================================

;==============================================================================
;	文字の書き出し	hl:文字数,文字列	de:vram adr offset
;==============================================================================
editor_put_str:
PROC
		ex		de,hl
		ld		bc,#3820
		add		hl,bc
		ex		de,hl
		ld		b,0
		ld		c,(hl)
		inc		hl
		jp		#005C
ENDP
macro	mPutText, tx, adr
	ld		hl,tx
	ld		de,adr
	call	editor_put_str
endm
;==============================================================================
;	HEXの書き出し	hl:vram adr offset	a:val
;==============================================================================
editor_put_number_hex:
proc
		ld		b,a
		ld		de,#3820
		add		hl,de
		call	#0053

		ld		a,b
		rrca
		rrca
		rrca
		rrca
		call	_put_a_hex
		ld		a,b
	_put_a_hex:
		and		#f
		add		a,'0'
		cp		'0'+10
		jr		c,_out
		add		a,7
	_out:
		out		(c),a
		ret
endp



;================================================================
;	キー入力
;================================================================
editor_get_key:
PROC
		ld		a,#7
		call	#0141
		bit		2,a
		jr		nz,_not_esc
		ld		a,#1b
		or		a
		ret
	_not_esc:
		ld		a,#6
		call	#0141
		bit		5,a
		jr		nz,_not_f1
	_trigger:
		ld		a,#20
		or		a
		ret
	_not_f1:

		call	#49f2
		ld		a,(#cdf1)
		or		a
		jr		nz,_trigger
	_stick:
		ld		a,(#cdee)
		inc		a
		rra
		ld		h,_stick_tbl>>8
		add		a,_stick_tbl&#ff
		ld		l,a
		ld		a,(hl)
		or		a
		ret		nz

		xor		a
		ld		hl,'0'
		ld		b,8
		call	_chk_mat
		ld		a,1
		ld		hl,'8'
		ld		b,2
		call	_chk_mat
		ld		a,2
		ld		hl,#3f00 + 'A'-6
		ld		b,8
		call	_chk_mat
		ld		a,3
		ld		hl,'C'
		ld		b,4
		call	_chk_mat
	;_key_is_off:
		xor		a
		ret

	_chk_mat:
		call	#0141
		or		h
	_lp_mat:
		rrca
		jr		nc,_ret_mat
		inc		l
		djnz	_lp_mat
		ret
	_ret_mat:
		pop		af
		ld		a,l
		or		a
		ret

	_stick_tbl:
		db		0,#1e,#1c,#1f,#1d
ENDP

;================================================================
;	編集系パラメータ初期化
;================================================================
init_menu_mode:
PROC
		xor		a
		ld		(cfg_cursor_pos),a

		ld		a,def_trapzone_mode
		ld		(cfg_trapzone_mode),a

		ld		a,(#002d)
		cp		#03
		sbc		a,a
		inc		a
		ld		(cfg_sound_driver),a

		xor		a
		ld		(cfg_bgm_test),a

		ld		a,def_map_type
		ld		(cfg_map_type),a

		ld		a,def_weight
		ld		(cfg_weight),a

		ld		a,def_pac_area
		ld		(cfg_pac_area),a

		ld		a,def_palette_cursor_pos
		ld		(cfg_palette_cursor_pos),a
		ld		ix,init_palette
		ld		iyl,1
		call	call_bank_6000

		ld		a,(pac_slot)
		inc		a
		jr		z,_not_pac
		ld		ix,pac_load_1stdata
		ld		iyl,bank_pac_access
		call	call_bank_pac_access
	_not_pac:
ENDP
;================================================================
;	コンフィグ全適用
;================================================================
apply_all_config:
PROC
		ld		a,(pac_slot)
		inc		a
		jr		z,_end_pac_save
		ld		a,(cfg_pac_area)
		ld		ix,pac_save
		ld		iyl,bank_pac_access
		call	call_bank_pac_access
	_end_pac_save:

		ld		a,(cfg_trapzone_mode)
		or		a
		jr		z,_set_trapzone_mode_end
		xor		a
		ld		(#cf1f),a	; 暗黒クリア
	_set_trapzone_mode_end:

		ld		ix,update_palette
		ld		iyl,1
		call	call_bank_6000

		jp		switch_sound_driver
ENDP




;================================================================
;	編集モード切り替え
;================================================================
editor_setup:
PROC
		; 画面クリア
		call	editor_cls
		call	editor_key_off_wait

		; キークリック音on
		ld		a,1
		ld		(CLIKSW),a
		ret
ENDP

editor_close:
PROC
		; キークリック音off
		xor		a
		ld		(CLIKSW),a

		call	editor_cls
		call	dumb_loop
		call	editor_key_off_wait
		ld		a,#ff
		ret
ENDP

; 画面クリア
editor_cls:
		xor		a
		ld		hl,#3820
		ld		bc,#03e0
		call	#0056
		ret

editor_key_off_wait:
PROC
		call	editor_get_key
		jr		nz,editor_key_off_wait
		ret
ENDP

dumb_loop:
PROC
		ld		bc,0
	_lp:
		cpi
		cpd
		ret		po
		jr		_lp
ENDP


;================================================================
;	メニューモード
;================================================================
menu_mode:
PROC
		di
		ld		hl,disp_flag	; メニュー表示フラグON
		set		1,(hl)
		ei

		; BGM記憶
		ld		a,(#ce95)
		push	af

		; メニューループ
		call	main_menu

		; BGM再現
		ld		a,(#ce95)
		ld		c,a
		pop		af
		cp		c
		jr		z,_end_load_bgm
		and		#7f
		ld		(#ce95),a
	_end_load_bgm:

		; コンフィグ適用
		call	apply_all_config

		; トラップゾーン表示切り替え
		call	switch_trapzone_bg

		di
		; メッセージ再描画
		ld		a,(#ce69)
		or		a
		jr		z,_no_msg
		push	af
		ld		a,#ff
		ld		(#ce69),a
		call	#5817
		pop		af
		ld		(#ce69),a
	_no_msg:
		ld		hl,disp_flag	; メニュー表示フラグOFF
		res		1,(hl)
		ei

		ret
ENDP


main_menu:
PROC
		call	editor_setup

		mPutText	tx_menu_0, 32*(menu_top_y_pos+ 0)+menu_key_x_pos
		mPutText	tx_menu_1, 32*(menu_top_y_pos+ 2)+menu_key_x_pos
		mPutText	tx_menu_2, 32*(menu_top_y_pos+ 4)+menu_key_x_pos
		mPutText	tx_menu_3, 32*(menu_top_y_pos+ 6)+menu_key_x_pos
		mPutText	tx_menu_4, 32*(menu_top_y_pos+ 8)+menu_key_x_pos
		mPutText	tx_menu_5, 32*(menu_top_y_pos+10)+menu_key_x_pos
		mPutText	tx_menu_6, 32*(menu_top_y_pos+12)+menu_key_x_pos
		mPutText	tx_menu_7, 32*(menu_top_y_pos+14)+menu_key_x_pos

		jr		_key_off_wait

	_main_loop:
		call	editor_get_key
		jr		z,_main_loop
		cp		#1b
		jp		z, _exit

		ld		hl,cfg_cursor_pos

		cp		#1e
		jr		nz,_not_up
		ld		a,(hl)
		dec		a
		jp		p,_not_top
		add		a,menu_cnt
	_not_top:
		ld		(hl),a
		jr		_key_off_wait
	_not_up:
		cp		#1f
		jr		nz,_not_down
		ld		a,(hl)
		inc		a
		cp		menu_cnt
		jr		c,_not_bottom
		xor		a
	_not_bottom:
		ld		(hl),a
		jr		_key_off_wait
	_not_down:
		ld		b,1
		cp		#20
		jr		z,_space_down
		cp		#1c
		jr		z,_lr_down
	_not_right:
		ld		b,#ff
		cp		#1d
		jr		z,_lr_down
	_key_off_wait:
		call	_update_disp
		call	editor_key_off_wait
		jr		_main_loop

	_space_down:
		ld		a,(hl)
		or		a
		jr		z,_exit
		cp		3
		jp		z,_exec_bgmtest
		cp		6
		jp		z,_exec_pacload
		cp		7
		jp		z,_exec_color_palette_editor
		jr		_key_off_wait

	_lr_down:
		ld		a,(hl)
		dec		a
		jr		z,_shift_trapzonecfg
		dec		a
		jr		z,_shift_soundcfg
		dec		a
		jr		z,_shift_bgmtest
		dec		a
		jr		z,_shift_maptypecfg
		dec		a
		jr		z,_shift_weightcfg
		dec		a
		jr		z,_shift_pacsave
		jr		_key_off_wait

	_exit:
		jp		editor_close

	_shift_trapzonecfg:
		ld		a,(cfg_trapzone_mode)
	_shift_trapzonecfg_ck
		add		a,b
		and		3
		cp		3
		jr		nc,_shift_trapzonecfg_ck
		ld		(cfg_trapzone_mode),a
		jr		_key_off_wait

	_shift_soundcfg:
		ld		a,(cfg_sound_driver)
		add		a,b
		and		1
		ld		(cfg_sound_driver),a
		call	stop_sound_driver
		jr		_key_off_wait

	_shift_bgmtest:
		ld		a,(cfg_bgm_test)
		add		a,b
		and		#f
		ld		(cfg_bgm_test),a
		jr		_key_off_wait
	_shift_maptypecfg:
		ld		a,(cfg_map_type)
		add		a,b
		and		1
		ld		(cfg_map_type),a
		call	reload_pcg_and_map
		jr		_key_off_wait
	_shift_weightcfg:
		ld		a,(cfg_weight)
		add		a,b
		and		#f
		ld		(cfg_weight),a
		jr		_key_off_wait
	_shift_pacsave:
		ld		a,(pac_slot)
		inc		a
		jr		z,_shift_pacsave_off
		ld		a,(cfg_pac_area)
		add		a,b
		jp		m,_shift_pacsave_8
		cp		9
		jr		c,_shift_pacsave_end
	_shift_pacsave_off:
		xor		a
	_shift_pacsave_end
		ld		(cfg_pac_area),a
		jp		_key_off_wait
	_shift_pacsave_8:
		ld		a,8
		jr		_shift_pacsave_end

	_exec_bgmtest:
		call	stop_sound_driver
		ld		a,(cfg_bgm_test)
		ld		(#ce95),a
		jp		_key_off_wait

	_exec_pacload:
		ld		a,(cfg_pac_area)
		or		a
		jp		z,_key_off_wait
		ld		ix,pac_has_data
		ld		iyl,bank_pac_access
		call	call_bank_pac_access
		jp		nz,_key_off_wait
		ld		hl,tx_pac_load_0
		ld		de,32*(menu_top_y_pos+12)+menu_val_x_pos
		call	editor_put_str
		call	stop_sound_driver
		ld		a,(cfg_pac_area)
		ld		ix,pac_load
		ld		iyl,bank_pac_access
		call	call_bank_pac_access
		ld		a,0
		call	dumb_weight
		call	apply_all_config
		call	reload_pcg_and_map
		jp		main_menu

	_exec_color_palette_editor:
		call	color_palette_editor
		jp		main_menu

	_update_disp:
	; カーソル表示
		ld		hl,menu_key_x_pos-2+(menu_top_y_pos-2+menu_cnt*2)*32 + #3820
		ld		b,menu_cnt
	_lp_put_cursor:
		ld		a,(cfg_cursor_pos)
		inc		a
		cp		b
		ld		a,#b6
		jr		z,_put_cursor
		ld		a,0
	_put_cursor:
		call	#004d
		ld		de,-2*32
		add		hl,de
		djnz	_lp_put_cursor

	; 潜る表示
		ld		de,32*(menu_top_y_pos+2)+menu_val_x_pos
		ld		a,(cfg_trapzone_mode)
		ld		hl,tx_trapzone_mode_0
		or		a
		jr		z,_put_trapzone_mode
		ld		hl,tx_trapzone_mode_1
		dec		a
		jr		z,_put_trapzone_mode
		ld		hl,tx_trapzone_mode_2
	_put_trapzone_mode:
		call	editor_put_str

	; サウンドドライバー表示
		ld		de,32*(menu_top_y_pos+4)+menu_val_x_pos
		ld		a,(cfg_sound_driver)
		or		a
		ld		hl,tx_sound_driver_0
		jr		z,_put_sound_driver
		ld		hl,tx_sound_driver_1
	_put_sound_driver:
		call	editor_put_str

	; BGMテスト表示
		ld		hl,32*(menu_top_y_pos+6)+menu_val_x_pos
		ld		a,(cfg_bgm_test)
		call	editor_put_number_hex

	; map type表示
		ld		de,32*(menu_top_y_pos+8)+menu_val_x_pos
		ld		a,(cfg_map_type)
		or		a
		ld		hl,tx_map_type_0
		jr		z,_put_map_type
		ld		hl,tx_map_type_1
	_put_map_type:
		call	editor_put_str

	; weight 表示
		ld		hl,32*(menu_top_y_pos+10)+menu_val_x_pos
		ld		a,(cfg_weight)
		call	editor_put_number_hex

	; pac save 表示
		ld		a,(cfg_pac_area)
		or		a
		jr		z,_put_pac_save_off
		ld		hl,32*(menu_top_y_pos+12)+menu_val_x_pos
		call	editor_put_number_hex
		push	bc
		ld		a,(cfg_pac_area)
		ld		ix,pac_has_data
		ld		iyl,bank_pac_access
		call	call_bank_pac_access
		ld		a,0
		jr		nz,_put_pac_data_exist
		ld		a,#b6
	_put_pac_data_exist:
		pop		bc
		out		(c),a
		ret
	_put_pac_save_off:
		ld		de,32*(menu_top_y_pos+12)+menu_val_x_pos
		ld		hl,tx_pac_save_0
		jp		editor_put_str

ENDP




;==============================================================================
; PCGとマップの再読込
;==============================================================================
reload_pcg_and_map:
		ld		a,(disp_flag)	; タイトル画面？
		rrca
		ret		c
		ld		a,(0xcddd)		; 非bossフラグ
		or		a
		ret		z

		; PCG再読込
		ld		ix,setup_pcg
		ld		iyl,1
		call	call_bank_6000

		; マップ再読込
		call	proc_48a0
		ld		ix,load_map
		ld		iyl,1
		ld		hl,(#cddf)
		push	hl					;現在位置保存
		call	call_bank_6000		;BANK1がほしい
		pop		hl					;現在位置復帰
		ld		(#cddf),hl
		ret

;==============================================================================
; トラップゾーンの表示/非表示
;==============================================================================
switch_trapzone_bg:
proc
		ld		hl,(#ce7c)
		push	hl

		ld		a,(cfg_trapzone_mode)
		cp		2
		ld		hl,#7c96		; 0が4つ
		jr		nz,_set_pcgno
		ld		hl,_trap_pcg
	_set_pcgno:
		ld		(#ce7c),hl

		ld		de,#cc9d
		ld		c,15
	_lp_y:
		ld		a,(cur_bg_y)
		rrca
		add		a,15
		sub		c
		sub		2
		and		#f
		ld		h,a
		xor		a
		rr		h
		rra
		rr		h
		rra
		ld		l,a

		ld		b,20
	_lp_x:
		ld		a,(de)
		cp		#96
		jr		nz,_next
	_put:
		push	hl
		push	de
		push	bc

		ex		de,hl
		ld		hl,(#ce7c)
		ld		bc,_ret_putchr16x16		; ret point
		push	bc							;
		push	bc			; dummy putchr16x16_でpop deされる
		ld		b,0
		jp		putchr16x16_
	_ret_putchr16x16:
		pop		bc
		pop		de
		pop		hl
	_next:
		inc		l
		inc		de
		djnz	_lp_x
		dec		c
		jr		nz,_lp_y

		pop		hl
		ld		(#ce7c),hl
		ret

	_trap_pcg:
		db		#b6,#b6,#b6,#b6
endp



;==============================================================================
;	文字
;==============================================================================
;JQZは使えない
menu_cnt			equ	8
menu_top_y_pos		equ	4
menu_key_x_pos		equ	5
menu_val_x_pos		equ	20

tx_menu_0:			db	 4,"EXIT"
tx_menu_1:			db	 9,"TRAP",0,"TILE"
tx_menu_2:			db	12,"SOUND",0,"DRIVER"
tx_menu_3:			db	 8,"BGM",0,"TEST"
tx_menu_4:			db	 3,"MAP"
tx_menu_5:			db	 6,"WEIGHT"
tx_menu_6:			db	 8,"PAC",0,"SRAM"
tx_menu_7:			db	13,"COLOR",0,"PALETTE"

tx_trapzone_mode_0:	db	4,"ON",0,0
tx_trapzone_mode_1:	db	4,"OFF",0
tx_trapzone_mode_2:	db	4,"SHOW"

tx_sound_driver_0:	db	6,"PSG",0,0,0
tx_sound_driver_1:	db	6,"ADXDRV"

tx_map_type_0:		db	3,"MSX"
tx_map_type_1:		db	3,"NES"

tx_pac_save_0:		db	3,"OFF"
tx_pac_load_0:		db	7,"LOADING"
