;===============================================================
;  MSX STAR SOLDIER
;
;	PAC save/load
;
;===============================================================

;================================================================
;	pacオープン/クローズ
;================================================================
pac_open:
		ld		a,(pac_slot)
		ld		h,#40
		call	#0024
		ld		hl,pac_ctrl_dat
		LD		(pac_ctrl_adr),hl
		ret

pac_close:
		ld		hl,#ffff
		LD		(pac_ctrl_adr),hl
pac_close2:
		ld		a,(my_slot)
		ld		h,#40
		jp		#0024


;================================================================
;	データヘッダ一致チェック
;================================================================
pac_is_match_header:
		ld		hl,pac_sram_area-#400
		add		a,a
		add		a,a
		add		a,h
		ld		h,a
pac_is_match_header_hl:
proc
		ld		de,pac_save_hdr
		ld		b,pac_save_hdr_len
	_lp:
		ld		a,(de)
		cp		(hl)
		ret		nz
		inc		hl		; zフラグ変化させない
		inc		de		; zフラグ変化させない
		djnz	_lp		; zフラグ変化させない
		ld		a,(hl)	; 優先エリア番号
		ret
endp

;================================================================
;	データの有無を確認	a:対象エリア番号	z:有 nz:無	a:優先エリア番号
;================================================================
pac_has_data:
proc
		push	af
		call	pac_open
		pop		af

		call	pac_is_match_header

		push	af
		call	pac_close
		pop		af
		ret
endp


;================================================================
;	優先エリアのデータを読み込む
;================================================================
pac_load_1stdata:
proc
		ld		b,8
	_lp:
		push	bc
		ld		a,b
		call	pac_has_data
		jr		z,_found
		pop		bc
		djnz	_lp
		ret
	_found:
		call	pac_load
		pop		af
		ret
endp


;================================================================
;	pacから読み込み	a:読み込みエリア番号
;================================================================
pac_load:
PROC
		push	af
		call	pac_open
		pop		af

		call	pac_is_match_header
		jr		nz, _exit

		; 優先番号
		inc		l

		; high score
		ld		de,#cf31
		ld		bc,6
		ldir
		; cfg
		ld		a,(hl)
		ld		(cfg_trapzone_mode),a
		inc		hl
		ld		a,(hl)
		ld		(cfg_sound_driver),a
		inc		hl
		ld		a,(hl)
		ld		(cfg_map_type),a
		inc		hl
		ld		a,(hl)
		ld		(cfg_weight),a
		inc		hl
		ld		a,(hl)
		ld		(cfg_pac_area),a
		inc		hl
		ld		de,cfg_palette_tbl
		ld		bc,32*16
		ldir
		ld		de,cfg_st2pal_tbl
		ld		bc,16*2
		ldir
	_exit:
		call	pac_close
		xor		a
		ret
ENDP


;================================================================
;	pacに保存	a:書き込みエリア番号
;================================================================
pac_save:
PROC
		push	af
		call	pac_open
		pop		af

		push	af
		or		a
		jr		z,_not_save

		; adr
		ld		de,pac_sram_area-#400
		add		a,a
		add		a,a
		add		a,d
		ld		d,a

		; header
		ld		hl,pac_save_hdr
		ld		bc,pac_save_hdr_len
		ldir

		; 優先番号
		inc		e

		; high score
		ld		hl,#cf31
		ld		bc,6
		ldir
		; cfg
		ld		a,(cfg_trapzone_mode)
		ld		(de),a
		inc		de
		ld		a,(cfg_sound_driver)
		ld		(de),a
		inc		de
		ld		a,(cfg_map_type)
		ld		(de),a
		inc		de
		ld		a,(cfg_weight)
		ld		(de),a
		inc		de
		ld		a,(cfg_pac_area)
		ld		(de),a
		inc		de
		ld		hl,cfg_palette_tbl
		ld		bc,32*16
		ldir
		ld		hl,cfg_st2pal_tbl
		ld		bc,16*2
		ldir

	_not_save:
		; 優先番号セット
		ld		b,8
	_lp:
		push	bc
		ld		a,b
		call	pac_is_match_header
		pop		bc
		jr		nz,_next
		pop		de
		ld		(hl),d
		push	de
	_next:
		djnz	_lp

		pop		af
		call	pac_close
		xor		a
		ret
ENDP



;================================================================
;
;================================================================
pac_save_hdr:
		db		"StarSoldierSS02"
pac_save_hdr_end:
pac_save_hdr_len	equ	pac_save_hdr_end-pac_save_hdr
