;==============================================================================
;
; MSX スターソルジャー
;
;	ROM初期処理
;	インターバンクコール
;
;==============================================================================
org		#4000
rom_id:
		db		0,0,0,0
apl_id:
		db		"KOS&SIRO"

;================================
;	ROM初期化処理
;================================
init_end:
		; 通常処理に戻る
		ld		a,0
		call	set_bank_4000

init_start:
proc
		ld		sp,#f000

		; turbo on
		ld		a,(#002d)
		cp		#03
		ld		a,#81
		call	nc,#0180

		ld		hl,apl_id
		ld		de,#cf29
		ld		b,8
	_ck_coldstart:
		ld		a,(de)
		cp		(hl)
		jr		nz,_coldstart
		inc		e
		inc		l
		djnz	_ck_coldstart
		ld		hl,#cf28
		ld		de,#cf28-1
		ld		bc,#0f28-1
		jr		_clear_ram
	_coldstart:
		ld		hl,#d000
		ld		de,#d000-1
		ld		bc,#1000-1
	_clear_ram:
		ld		(hl),0
		lddr
		ld		hl,#d000
		ld		de,#d001
		ld		bc,#17ff
		ld		(hl),0
		ldir

		ld		hl,apl_id
		ld		de,#cf29
		ld		bc,8
		ldir

		; where am i
		call	where_am_i
		ld		(my_slot),a
		;ページ2もこのROMにする
		ld		h,#80
		call	ENASLT

		; バンク切り替え処理をメモリに展開
		ld		hl,interbank_src
		ld		de,interbank_org
		ld		bc,interbank_end-interbank_org
		ldir

		; 割り込み処理をメモリに展開
		ld		hl,my_hkey_src
		ld		de,my_hkey_org
		ld		bc,my_hkey_end-my_hkey_org
		ldir

		; バンク初期化
		ld		a,bank_rominit
		call	set_bank_4000
		ld		a,1
		call	set_bank_6000
		inc		a
		call	set_bank_8000
		inc		a
		call	set_bank_A000

		; PACスロット検索
		ld		ix,is_pac_slot
		call	search_slots
		ld		(pac_slot),a

		; OPLLスロット検索
		ld		ix,is_aprl_slot
		call	search_slots
		ld		(opll_slot),a
		inc		a
		jr		nz,_end_opll
		ld		ix,is_opll_slot
		call	search_slots
		ld		(opll_slot),a
		; fm-pac opll有効化
		ld		hl,#7ff6
		push	af
		push	hl
		call	#000c	;RDSLT
		or		1
		ld		e,a
		pop		hl
		pop		af
		call	#0014	;WRSLT
	_end_opll:
		; RAMスロット検索
		ld		h,#40
		call	search_ram_slot
		ld		(ram_slot_4000),a
		ld		h,#80
		call	search_ram_slot
		ld		(ram_slot_8000),a

		; コンフィグメニュー初期化
		ld		ix,init_menu_mode
		ld		iyl,bank_menu
		call	call_bank_6000

		; ADXドライバ初期化
		ld		ix,ADXIF_setup
		ld		iyl,bank_adxif
		call	call_bank_4000

		jp		init_end
endp

;===============================================================
;	スロット検索
;===============================================================
where_am_i:
		call	RSLREG
		rrca
		rrca
		and		#3
		ld		c,a
		ld		b,0
		ld		hl,EXPTBL
		add		hl,bc
		ld		a,(hl)
		and		#80
		or		c
		ld		c,a
		inc		hl
		inc		hl
		inc		hl
		inc		hl
		ld		a,(hl)
		and		#c
		or		c
		ret

search_ram_slot:
		ld		ix,is_ram_slot
search_slots:
		xor		a
search_slots_next:
proc
		ld		iy,EXPTBL
	_lp:
		or		(iy+0)
	_lp_subslot:
		push	af
		push	hl
		push	ix
		push	iy
		call	call_ix
		pop		iy
		pop		ix
		pop		hl
		jr		z,_found
		pop		af

		add		a,#04
		jp		p,_next
		bit		4,a
		jr		z,_lp_subslot
	_next:
		inc		iy
		inc		a
		and		#03
		jr		nz,_lp
		ld		a,#ff
		ret
	_found:
		pop		af
		ret
endp

is_ram_slot:
proc
		ld		c,#04
	_loop_c:
		ld		l,#10
		ld		b,#0f
	_loop_b:
		call	is_slot_writable
		jr		nc,_not
		dec		l
		djnz	_loop_b
		ld		de,#400
		add		hl,de
		dec		c
		jr		nz,_loop_c
		xor		a
		ret
	_not:
		xor		a
		dec		a
		ret
endp

;	書き込み可？	hl:アドレス a:slot	cf:可　nc:不可
is_slot_writable:
proc
		push	bc
		push	de

		push	af
		call	#000c	;RDSLT
		cpl
		ld		e,a
		pop		af

		push	af
		call	#0014	;WRSLT
		pop		af

		push	af
		push	de
		call	#000c	;RDSLT
		pop		de

		cp		e
		jr		nz,_not
		cpl
		ld		e,a
		pop		af

		push	af
		call	#0014	;WRSLT
		pop		af

		scf
		jr		_ret

	_not:
		pop		af
		or		a
	_ret:
		pop		de
		pop		bc
		ret
endp

is_aprl_slot:
		ld		de,sig_aprl
		ld		hl,#4018
		ld		b,8
		jr		is_opll_slot_loop
is_opll_slot:
		ld		de,sig_opll
		ld		hl,#4018+4
		ld		b,4
is_opll_slot_loop:
proc
		push	af
		push	bc
		push	de
		call	#000c	; RDSLT
		pop		de
		pop		bc
		ex		de,hl
		cp		(hl)
		ex		de,hl
		jr		nz,_not
		pop		af
		inc		l
		inc		de
		djnz	is_opll_slot_loop
		xor		a
		ret
	_not:
		pop		af
		xor		a
		dec		a
		ret
endp
sig_aprl:
		db		"APRL"
sig_opll:
		db		"OPLL"

is_pac_slot:
proc
		ld		hl,#ffff
		call	_chk_pac
		jr		c,_not
		ld		hl,pac_ctrl_dat
		call	_chk_pac
		jr		nc,_not
		xor		a
		ret
	_not:
		xor		a
		dec		a
		ret
	_chk_pac:
		call	_rdDEfmPacCtrl
		push	de				; store PacCtrl data
		call	_wtHLtoPacCtrl
		ld		hl,#4010
		call	is_slot_writable
		pop		hl				; restore PacCtrl data
		call	_wtHLtoPacCtrl
		ret
		
	_rdDEfmPacCtrl:
		push	hl
		ld		hl,pac_ctrl_adr
		push	af
		call	#000c	; RDSLT
		ld		e,a
		pop		af
		inc		hl
		push	af
		push	de
		call	#000c	; RDSLT
		pop		de
		ld		d,a
		pop		af
		pop		hl
		ret
	_wtHLtoPacCtrl:
		push	de
		push	hl
		push	af
		ld		e,l
		ld		hl,pac_ctrl_adr
		call	#0014	; WRSLT
		pop		af
		pop		hl
		push	hl
		push	af
		ld		e,h
		ld		hl,pac_ctrl_adr+1
		call	#0014	; WRSLT
		pop		af
		pop		hl
		pop		de
		ret
endp

;===============================================================
; インターバンク呼び出し
;	set_bank_4000	a:bank番号
;	set_bank_6000	a:bank番号
;	set_bank_8000	a:bank番号
;	set_bank_a000	a:bank番号
;	call_bank_4000	iyl:bank番号 ix:呼び出し先アドレス
;	call_bank_6000	iyl:bank番号 ix:呼び出し先アドレス
;	call_bank_8000	iyl:bank番号 ix:呼び出し先アドレス
;	call_bank_a000	iyl:bank番号 ix:呼び出し先アドレス
;===============================================================
interbank_src:
org		interbank_org

ld_hl_pc:	pop		hl
call_hl:	jp		(hl)
call_ix:	jp		(ix)
call_iy:	jp		(iy)

MACRO	mBankFunk,	_adr
	set_bank_##_adr:
		ld		(current_bank_##_adr),a
		ld		(bank_sw_##_adr),a
		ret
	call_bank_##_adr:
		push	af
		ld		a,(current_bank_##_adr)
		ld		iyh,a
		ld		a,iyl
		call	set_bank_##_adr
		pop		af
		push	iy
		call	call_ix
		pop		iy
		push	af
		ld		a,iyh
		call	set_bank_##_adr
		pop		af
		ret
ENDM

IRP	adr, #4000, #6000, #8000, #A000
		mBankFunk	adr
ENDM

interbank_end:
org		interbank_src + interbank_end-interbank_org
